@@ -18,8 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef __DFU_H__
-#define __DFU_H__
+#ifndef DFU_H
+#define DFU_H
 
 #include <libusb.h>
 #include "usb_dfu.h"
@@ -91,14 +91,15 @@ struct dfu_status {
 };
 
 struct dfu_if {
-    u_int16_t vendor;
-    u_int16_t product;
-    u_int8_t configuration;
-    u_int8_t interface;
-    u_int8_t altsetting;
+    uint16_t vendor;
+    uint16_t product;
+    uint16_t bcdDevice;
+    uint8_t configuration;
+    uint8_t interface;
+    uint8_t altsetting;
     unsigned char *alt_name;
     int bus;
-    u_int8_t devnum;
+    uint8_t devnum;
     const char *path;
     unsigned int flags;
     unsigned int count;
@@ -129,9 +130,10 @@ int dfu_get_state( libusb_device_handle *device,
 int dfu_abort( libusb_device_handle *device,
                const unsigned short interface );
 
-char* dfu_state_to_string( int state );
+const char *dfu_state_to_string( int state );
 
-const char *dfu_status_to_string(int status);
+const char *dfu_status_to_string( int status );
 
 int debug;
-#endif
+
+#endif /* DFU_H */
