@@ -23,14 +23,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
 #include <string.h>
 #include <libusb.h>
 
-#include "config.h"
+#include "portable.h"
 #include "dfu.h"
 #include "usb_dfu.h"
 #include "dfu_file.h"
@@ -61,7 +58,7 @@ int dfuload_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file)
 			ret = rc;
 			goto out_free;
 		}
-		write_rc = write(file.fd, buf, rc);
+		write_rc = fwrite(buf, 1, rc, file.filep);
 		if (write_rc < rc) {
 			fprintf(stderr, "Short file write: %s\n",
 				strerror(errno));
@@ -108,9 +105,7 @@ int dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file)
 	if (bytes_per_hash == 0)
 		bytes_per_hash = 1;
 	printf("bytes_per_hash=%u\n", bytes_per_hash);
-#if 0
-	read(file.fd, DFU_HDR);
-#endif
+
 	printf("Copying data from PC to DFU device\n");
 	printf("Starting download: [");
 	fflush(stdout);
@@ -124,7 +119,7 @@ int dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file)
 			chunk_size = bytes_left;
 		else
 			chunk_size = xfer_size;
-		ret = read(file.fd, buf, chunk_size);
+		ret = fread(buf, 1, chunk_size, file.filep);
 		if (ret < 0) {
 			perror(file.name);
 			goto out_free;
@@ -149,9 +144,9 @@ int dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file)
 
 			/* Wait while device executes flashing */
 			if (quirks & QUIRK_POLLTIMEOUT)
-				usleep(DEFAULT_POLLTIMEOUT * 1000);
+				milli_sleep(DEFAULT_POLLTIMEOUT);
 			else
-				usleep(dst.bwPollTimeout * 1000);
+				milli_sleep(dst.bwPollTimeout);
 
 		} while (1);
 		if (dst.bStatus != DFU_STATUS_OK) {
@@ -193,7 +188,7 @@ get_status:
 		dfu_state_to_string(dst.bState), dst.bStatus,
 		dfu_status_to_string(dst.bStatus));
 	if (!(quirks & QUIRK_POLLTIMEOUT))
-		usleep(dst.bwPollTimeout * 1000);
+		milli_sleep(dst.bwPollTimeout);
 
 	/* FIXME: deal correctly with ManifestationTolerant=0 / WillDetach bits */
 	switch (dst.bState) {
@@ -201,19 +196,12 @@ get_status:
 	case DFU_STATE_dfuMANIFEST:
 		/* some devices (e.g. TAS1020b) need some time before we
 		 * can obtain the status */
-		sleep(1);
+		milli_sleep(1000);
 		goto get_status;
 		break;
 	case DFU_STATE_dfuIDLE:
 		break;
 	}
-#if 0
-	printf("Resetting USB...\n");
-	if (usb_reset(dif->dev_handle) < 0) {
-		fprintf(stderr, "error resetting after download: %s\n",
-			usb_strerror());
-	}
-#endif
 	printf("Done!\n");
 
 out_free:
