@@ -1,7 +1,7 @@
-#ifndef _SAM7DFU_H
-#define _SAM7DFU_H
+#ifndef DFU_LOAD_H
+#define DFU_LOAD_H
 
 int dfuload_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file);
 int dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file);
 
-#endif
+#endif /* DFU_LOAD_H */
