@@ -1,7 +1,8 @@
 /* This implements the ST Microsystems DFU extensions (DfuSe)
- * as per the DfuSe 1.1a specification (Document UM0391)
+ * as per the DfuSe 1.1a specification (ST documents AN3156, AN2606)
+ * The DfuSe file format is described in ST document UM0391.
  *
- * (C) 2010-2011 Tormod Volden <debian.tormod@gmail.com>
+ * (C) 2010-2012 Tormod Volden <debian.tormod@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,35 +21,99 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include <errno.h>
 #include <string.h>
-#include <sys/stat.h>
 
-#include "config.h"
+#include "portable.h"
 #include "dfu.h"
 #include "usb_dfu.h"
 #include "dfu_file.h"
 #include "dfuse.h"
 #include "dfuse_mem.h"
 
-/* ugly hack for Win32 */
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
 #define DFU_TIMEOUT 5000
 
 extern int verbose;
 static unsigned int last_erased = 0;
 static struct memsegment *mem_layout;
+static unsigned int dfuse_address = 0;
+static unsigned int dfuse_length = 0;
+static int dfuse_force = 0;
+static int dfuse_leave = 0;
+static int dfuse_unprotect = 0;
+static int dfuse_mass_erase = 0;
 
 unsigned int quad2uint(unsigned char *p)
 {
 	return (*p + (*(p + 1) << 8) + (*(p + 2) << 16) + (*(p + 3) << 24));
 }
 
+void dfuse_parse_options(const char *options)
+{
+	char *end;
+	const char *endword;
+	unsigned int number;
+
+	/* address, possibly empty, must be first */
+	if (*options != ':') {
+		endword = strchr(options, ':');
+		if (!endword)
+			endword = options + strlen(options); /* GNU strchrnul */
+
+		number = strtoul(options, &end, 0);
+		if (end == endword) {
+			dfuse_address = number;
+		} else {
+			fprintf(stderr, "Error: Invalid dfuse address: "
+				"%s\n", options);
+			exit(2);
+		}
+		options = endword;
+	}
+
+	while (*options) {
+		if (*options == ':') {
+			options++;
+			continue;
+		}
+		endword = strchr(options, ':');
+		if (!endword)
+			endword = options + strlen(options);
+
+		if (!strncmp(options, "force", endword - options)) {
+			dfuse_force++;
+			options += 5;
+			continue;
+		}
+		if (!strncmp(options, "leave", endword - options)) {
+			dfuse_leave = 1;
+			options += 5;
+			continue;
+		}
+		if (!strncmp(options, "unprotect", endword - options)) {
+			dfuse_unprotect = 1;
+			options += 9;
+			continue;
+		}
+		if (!strncmp(options, "mass-erase", endword - options)) {
+			dfuse_mass_erase = 1;
+			options += 10;
+			continue;
+		}
+
+		/* any valid number is interpreted as upload length */
+		number = strtoul(options, &end, 0);
+		if (end == endword) {
+			dfuse_length = number;
+		} else {
+			fprintf(stderr, "Error: Invalid dfuse modifier: "
+				"%s\n", options);
+			exit(2);
+		}
+		options = endword;
+	}
+}
+
 /* DFU_UPLOAD request for DfuSe 1.1a */
 int dfuse_upload(struct dfu_if *dif, const unsigned short length,
 		 unsigned char *data, unsigned short transaction)
@@ -132,6 +197,9 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 	} else if (command == MASS_ERASE) {
 		buf[0] = 0x41;	/* Mass erase command when length = 1 */
 		length = 1;
+	} else if (command == READ_UNPROTECT) {
+		buf[0] = 0x92;
+		length = 1;
 	} else {
 		fprintf(stderr, "Error: Non-supported special command %d\n",
 			command);
@@ -157,18 +225,26 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 		exit(1);
 	}
 	/* wait while command is executed */
-	usleep(dst.bwPollTimeout * 1000);
+	if (verbose)
+		printf("   Poll timeout %i ms\n", dst.bwPollTimeout);
+	milli_sleep(dst.bwPollTimeout);
+
+	if (command == READ_UNPROTECT)
+		return ret;
 
 	ret = dfu_get_status(dif->dev_handle, dif->interface, &dst);
 	if (ret < 0) {
 		fprintf(stderr, "Error during second get_status\n");
+		printf("state(%u) = %s, status(%u) = %s\n", dst.bState,
+		       dfu_state_to_string(dst.bState), dst.bStatus,
+		       dfu_status_to_string(dst.bStatus));
 		exit(1);
 	}
 	if (dst.bStatus != DFU_STATUS_OK) {
 		fprintf(stderr, "Error: Command not correctly executed\n");
 		exit(1);
 	}
-	usleep(dst.bwPollTimeout * 1000);
+	milli_sleep(dst.bwPollTimeout);
 
 	ret = dfu_abort(dif->dev_handle, dif->interface);
 	if (ret < 0) {
@@ -184,12 +260,12 @@ int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 		fprintf(stderr, "Error: Failed to enter idle state on abort\n");
 		exit(1);
 	}
-	usleep(dst.bwPollTimeout * 1000);
+	milli_sleep(dst.bwPollTimeout);
 	return ret;
 }
 
 int dfuse_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
-		    unsigned int dfuse_address)
+		    const char *dfuse_options)
 {
 	int total_bytes = 0;
 	int upload_limit = 0;
@@ -201,6 +277,10 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
 	if (!buf)
 		return -ENOMEM;
 
+	if (dfuse_options)
+		dfuse_parse_options(dfuse_options);
+	if (dfuse_length)
+		upload_limit = dfuse_length;
 	if (dfuse_address) {
 		struct memsegment *segment;
 
@@ -211,18 +291,24 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
 			exit(1);
 		}
 		segment = find_segment(mem_layout, dfuse_address);
-		if (!segment || !(segment->memtype & DFUSE_READABLE)) {
+		if (!dfuse_force &&
+		    (!segment || !(segment->memtype & DFUSE_READABLE))) {
 			fprintf(stderr,
 				"Error: Page at 0x%08x is not readable\n",
 				dfuse_address);
 			exit(1);
 		}
-		upload_limit = segment->end - dfuse_address + 1;
-		printf("Limiting upload to end of memory segment, %i bytes\n",
-		       upload_limit);
+		if (!upload_limit) {
+			upload_limit = segment->end - dfuse_address + 1;
+			printf("Limiting upload to end of memory segment, "
+			       "%i bytes\n", upload_limit);
+		}
 		dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
 	} else {
-		upload_limit = 0x4000;	/* Should be safe for most devices */
+		/* Boot loader decides the start address, unknown to us */
+		/* Use a short length to lower risk of running out of bounds */
+		if (!upload_limit)
+			upload_limit = 0x4000;
 		printf("Limiting default upload to %i bytes\n", upload_limit);
 	}
 
@@ -233,12 +319,16 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
 	transaction = 2;
 	while (1) {
 		int rc, write_rc;
+
+		/* last chunk can be smaller than original xfer_size */
+		if (upload_limit - total_bytes < xfer_size)
+			xfer_size = upload_limit - total_bytes;
 		rc = dfuse_upload(dif, xfer_size, buf, transaction++);
 		if (rc < 0) {
 			ret = rc;
 			goto out_free;
 		}
-		write_rc = write(file.fd, buf, rc);
+		write_rc = fwrite(buf, 1, rc, file.filep);
 		if (write_rc < rc) {
 			fprintf(stderr, "Short file write: %s\n",
 				strerror(errno));
@@ -284,9 +374,13 @@ int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 			fprintf(stderr, "Error during download get_status\n");
 			return ret;
 		}
-		usleep(dst.bwPollTimeout * 1000);
+		milli_sleep(dst.bwPollTimeout);
 	} while (dst.bState != DFU_STATE_dfuDNLOAD_IDLE &&
-		 dst.bState != DFU_STATE_dfuERROR);
+		 dst.bState != DFU_STATE_dfuERROR &&
+		 dst.bState != DFU_STATE_dfuMANIFEST);
+
+	if (dst.bState == DFU_STATE_dfuMANIFEST)
+			printf("Transitioning to dfuMANIFEST state\n");
 
 	if (dst.bStatus != DFU_STATUS_OK) {
 		printf(" failed!\n");
@@ -337,7 +431,7 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 			chunk_size = dwElementSize - p;
 
 		/* Erase only for flash memory downloads */
-		if (segment->memtype & DFUSE_ERASABLE) {
+		if ((segment->memtype & DFUSE_ERASABLE) && !dfuse_mass_erase) {
 			/* erase all involved pages */
 			for (erase_address = address;
 			     erase_address < address + chunk_size;
@@ -351,19 +445,24 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 			if (((address + chunk_size - 1) & ~(page_size - 1)) !=
 			    (last_erased & ~(page_size - 1))) {
 				if (verbose > 2)
-					printf(" Chunk wraps over to next page"
-					       "\n");
+					printf(" Chunk extends into next page,"
+					       " erase it as well\n");
 				dfuse_special_command(dif,
 						      address + chunk_size - 1,
 						      ERASE_PAGE);
 			}
 		}
 
-		if (verbose)
+		if (verbose) {
 			printf(" Download from image offset "
 			       "%08x to memory %08x-%08x, size %i\n",
 			       p, address, address + chunk_size - 1,
 			       chunk_size);
+		} else {
+			printf(".");
+			fflush(stdout);
+		}
+		
 		dfuse_special_command(dif, address, SET_ADDRESS);
 
 		/* transaction = 2 for no address offset */
@@ -374,6 +473,8 @@ int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddress,
 			return -EINVAL;
 		}
 	}
+	if (!verbose)
+		printf("\n"); /* terminate line of dots */
 	return 0;
 }
 
@@ -389,16 +490,15 @@ int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 
 	dwElementAddress = start_address;
 	dwElementSize = file.size;
-	if (verbose)
-		printf("Uploading to address = 0x%08x, size = %i\n",
-		       dwElementAddress, dwElementSize);
+	printf("Downloading to address = 0x%08x, size = %i\n",
+	       dwElementAddress, dwElementSize);
 
 	data = malloc(dwElementSize);
 	if (!data) {
 		fprintf(stderr, "Could not allocate data buffer\n");
 		return -ENOMEM;
 	}
-	ret = read(file.fd, data, dwElementSize);
+	ret = fread(data, 1, dwElementSize, file.filep);
 	read_bytes += ret;
 	if (ret < dwElementSize) {
 		fprintf(stderr, "Could not read data\n");
@@ -412,9 +512,10 @@ int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 		goto out_free;
 
 	if (read_bytes != file.size) {
-		fprintf(stderr, "Warning: Read %i bytes, file size %i\n",
-			read_bytes, (int)file.size);
+		fprintf(stderr, "Warning: Read %i bytes, file size %li\n",
+			read_bytes, file.size);
 	}
+	printf("File downloaded successfully\n");
 	ret = read_bytes;
 
  out_free:
@@ -447,7 +548,7 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 		return -EINVAL;
 	}
 
-	ret = read(file.fd, dfuprefix, sizeof(dfuprefix));
+	ret = fread(dfuprefix, 1, sizeof(dfuprefix), file.filep);
 	if (ret < (int)sizeof(dfuprefix)) {
 		fprintf(stderr, "Could not read DfuSe header\n");
 		return -EIO;
@@ -467,7 +568,7 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 
 	for (image = 1; image <= bTargets; image++) {
 		printf("parsing DFU image %i\n", image);
-		ret = read(file.fd, targetprefix, sizeof(targetprefix));
+		ret = fread(targetprefix, 1, sizeof(targetprefix), file.filep);
 		read_bytes += ret;
 		if (ret < sizeof(targetprefix)) {
 			fprintf(stderr, "Could not read DFU header\n");
@@ -490,8 +591,8 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 			       " to download this image!\n");
 		for (element = 1; element <= dwNbElements; element++) {
 			printf("parsing element %i, ", element);
-			ret =
-			    read(file.fd, elementheader, sizeof(elementheader));
+			ret = fread(elementheader, 1, sizeof(elementheader),
+				    file.filep);
 			read_bytes += ret;
 			if (ret < sizeof(elementheader)) {
 				fprintf(stderr,
@@ -518,7 +619,7 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 					"Could not allocate data buffer\n");
 				return -ENOMEM;
 			}
-			ret = read(file.fd, data, dwElementSize);
+			ret = fread(data, 1, dwElementSize, file.filep);
 			read_bytes += ret;
 			if (ret < dwElementSize) {
 				fprintf(stderr, "Could not read data\n");
@@ -545,7 +646,7 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 		fprintf(stderr, "Could not allocate data buffer for suffix\n");
 		return -ENOMEM;
 	}
-	ret = read(file.fd, data, file.suffixlen);
+	ret = fread(data, 1, file.suffixlen, file.filep);
 	free(data);
 	if (ret < file.suffixlen) {
 		fprintf(stderr, "Could not read through suffix\n");
@@ -554,8 +655,8 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 	read_bytes += ret;
 
 	if (read_bytes != file.size) {
-		fprintf(stderr, "Warning: Read %i bytes, file size %i\n",
-			read_bytes, (int)file.size);
+		fprintf(stderr, "Warning: Read %i bytes, file size %li\n",
+			read_bytes, file.size);
 	}
 
 	printf("done parsing DfuSe file\n");
@@ -563,27 +664,67 @@ int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 }
 
 int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file file,
-		    unsigned int address)
+		    const char *dfuse_options)
 {
+	int ret;
+
+	if (dfuse_options)
+		dfuse_parse_options(dfuse_options);
 	mem_layout = parse_memory_layout((char *)dif->alt_name);
 	if (!mem_layout) {
 		fprintf(stderr, "Error: Failed to parse memory layout\n");
 		exit(1);
 	}
-	if (address) {
+	if (dfuse_unprotect) {
+		if (!dfuse_force) {
+			fprintf(stderr, "Error: The read unprotect command "
+				"will erase the flash memory\n"
+				"and can only be used with force\n");
+			exit(1);
+		}
+		dfuse_special_command(dif, 0, READ_UNPROTECT);
+		printf("Device disconnects, erases flash and resets now\n");
+		exit(0);
+	}
+	if (dfuse_mass_erase) {
+		if (!dfuse_force) {
+			fprintf(stderr, "Error: The mass erase command "
+				"can only be used with force\n");
+			exit(1);
+		}
+		printf("Performing mass erase, this can take a moment\n");
+		dfuse_special_command(dif, 0, MASS_ERASE);
+	}
+	if (dfuse_address) {
 		if (file.bcdDFU == 0x11a) {
 			fprintf(stderr, "Error: This is a DfuSe file, not "
 				"meant for raw download\n");
 			return -EINVAL;
 		}
-		return dfuse_do_bin_dnload(dif, xfer_size, file, address);
+		ret = dfuse_do_bin_dnload(dif, xfer_size, file, dfuse_address);
 	} else {
 		if (file.bcdDFU != 0x11a) {
 			fprintf(stderr, "Error: Only DfuSe file version 1.1a "
 				"is supported\n");
+			fprintf(stderr, "(for raw binary download, use the "
+				"--dfuse-address option)\n");
 			return -EINVAL;
 		}
-		return dfuse_do_dfuse_dnload(dif, xfer_size, file);
+		ret = dfuse_do_dfuse_dnload(dif, xfer_size, file);
 	}
 	free_segment_list(mem_layout);
+
+	if (dfuse_leave) {
+		int ret2;
+		struct dfu_status dst;
+
+		dfuse_dnload_chunk(dif, NULL, 0, 2); /* Zero-size */
+		ret2 = dfu_get_status(dif->dev_handle, dif->interface, &dst);
+		if (ret2 < 0)
+			fprintf(stderr, "Error during download get_status\n");
+		if (verbose)
+			printf("bState = %i and bStatus = %i\n",
+			       dst.bState, dst.bStatus);
+	}
+	return ret;
 }
