@@ -23,38 +23,26 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <string.h>
 #include <getopt.h>
 #include <libusb.h>
 #include <errno.h>
-#include <fcntl.h>
 
+#include "portable.h"
 #include "dfu.h"
 #include "usb_dfu.h"
 #include "dfu_file.h"
 #include "dfu_load.h"
 #include "dfuse.h"
 #include "quirks.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #ifdef HAVE_USBPATH_H
 #include <usbpath.h>
 #endif
 
-/* ugly hack for Win32 */
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
 int debug;
 int verbose = 0;
 
-/* If we really have to guess (non-compliant devices) */
-#define DEFAULT_TRANSFER_SIZE 1024
-
 /* USB string descriptor should contain max 126 UTF-16 characters
  * but 253 would even accomodate any UTF-8 encoding */
 #define MAX_DESC_STR_LEN 253
@@ -76,10 +64,14 @@ static int find_dfu_if(libusb_device *dev,
 	int rc;
 
 	memset(dfu_if, 0, sizeof(*dfu_if));
-	libusb_get_device_descriptor(dev, &desc);
+	rc = libusb_get_device_descriptor(dev, &desc);
+	if (rc)
+		return rc;
 	for (cfg_idx = 0; cfg_idx < desc.bNumConfigurations;
 	     cfg_idx++) {
-		libusb_get_config_descriptor(dev, cfg_idx, &cfg);
+		rc = libusb_get_config_descriptor(dev, cfg_idx, &cfg);
+		if (rc)
+			return rc;
 		/* in some cases, noticably FreeBSD if uid != 0,
 		 * the configuration descriptors are empty */
 		if (!cfg)
@@ -97,10 +89,9 @@ static int find_dfu_if(libusb_device *dev,
 				if (intf->bInterfaceClass == 0xfe &&
 				    intf->bInterfaceSubClass == 1) {
 					dfu_if->dev = dev;
-					dfu_if->vendor =
-						desc.idVendor;
-					dfu_if->product =
-						desc.idProduct;
+					dfu_if->vendor = desc.idVendor;
+					dfu_if->product = desc.idProduct;
+					dfu_if->bcdDevice = desc.bcdDevice;
 					dfu_if->configuration = cfg->
 							bConfigurationValue;
 					dfu_if->interface =
@@ -128,7 +119,7 @@ static int find_dfu_if(libusb_device *dev,
 
 static int _get_first_cb(struct dfu_if *dif, void *v)
 {
-	struct dfu_if *v_dif = v;
+	struct dfu_if *v_dif = (struct dfu_if*) v;
 
 	/* Copy everything except the device handle.
 	 * This depends heavily on this member being last! */
@@ -146,7 +137,7 @@ static int get_first_dfu_if(struct dfu_if *dif)
 
 static int _check_match_cb(struct dfu_if *dif, void *v)
 {
-	struct dfu_if *v_dif = v;
+	struct dfu_if *v_dif = (struct dfu_if*) v;
 
 	if (v_dif->flags & DFU_IFF_IFACE && 
 	    dif->interface != v_dif->interface)
@@ -165,7 +156,7 @@ static int get_matching_dfu_if(struct dfu_if *dif)
 
 static int _count_match_cb(struct dfu_if *dif, void *v)
 {
-	struct dfu_if *v_dif = v;
+	struct dfu_if *v_dif = (struct dfu_if*) v;
 
 	if (v_dif->flags & DFU_IFF_IFACE && 
 	    dif->interface != v_dif->interface)
@@ -204,7 +195,8 @@ static int get_alt_name(struct dfu_if *dfu_if, unsigned char *name)
 	ret = -1;
 	if (alt_name_str_idx) {
 		if (!dfu_if->dev_handle)
-			libusb_open(dfu_if->dev, &dfu_if->dev_handle);
+			if (libusb_open(dfu_if->dev, &dfu_if->dev_handle))
+				dfu_if->dev_handle = NULL;
 		if (dfu_if->dev_handle)
 			ret = libusb_get_string_descriptor_ascii(
 					dfu_if->dev_handle, alt_name_str_idx,
@@ -264,7 +256,7 @@ static int alt_by_name(struct dfu_if *dfu_if, void *v)
 
 static int _count_cb(struct dfu_if *dif, void *v)
 {
-	int *count = v;
+	int *count = (int*) v;
 
 	(*count)++;
 
@@ -299,7 +291,8 @@ static int iterate_dfu_devices(libusb_context *ctx, struct dfu_if *dif,
 		    (libusb_get_bus_number(dev) != dif->bus ||
 		     libusb_get_device_address(dev) != dif->devnum))
 			continue;
-		libusb_get_device_descriptor(dev, &desc);
+		if (libusb_get_device_descriptor(dev, &desc))
+			continue;
 		if (dif && (dif->flags & DFU_IFF_VENDOR) &&
 		    desc.idVendor != dif->vendor)
 			continue;
@@ -322,7 +315,7 @@ static int iterate_dfu_devices(libusb_context *ctx, struct dfu_if *dif,
 
 static int found_dfu_device(struct libusb_device *dev, void *user)
 {
-	struct dfu_if *dif = user;
+	struct dfu_if *dif = (struct dfu_if*) user;
 
 	dif->dev = dev;
 	return 1;
@@ -338,7 +331,7 @@ static int get_first_dfu_device(libusb_context *ctx, struct dfu_if *dif)
 
 static int count_one_dfu_device(struct libusb_device *dev, void *user)
 {
-	int *num = user;
+	int *num = (int*) user;
 
 	(*num)++;
 	return 0;
@@ -355,7 +348,7 @@ static int count_dfu_devices(libusb_context *ctx, struct dfu_if *dif)
 }
 
 
-static void parse_vendprod(u_int16_t *vendor, u_int16_t *product,
+static void parse_vendprod(uint16_t *vendor, uint16_t *product,
 			   const char *str)
 {
 	const char *colon;
@@ -437,12 +430,17 @@ static int usb_get_any_descriptor(struct libusb_device_handle *dev_handle,
 				  uint8_t desc_index,
 				  unsigned char *resbuf, int res_len)
 {
-	struct libusb_device *dev = libusb_get_device(dev_handle);
+	struct libusb_device *dev;
 	struct libusb_config_descriptor *config;
 	int ret;
 	uint16_t conflen;
 	unsigned char *cbuf;
 
+	dev = libusb_get_device(dev_handle);
+	if (!dev) {
+		fprintf(stderr, "Error: Broken device handle\n");
+		return -1;
+	}
 	/* Get the total length of the configuration descriptors */
 	ret = libusb_get_active_config_descriptor(dev, &config);
 	if (ret == LIBUSB_ERROR_NOT_FOUND) {
@@ -531,18 +529,19 @@ static int get_cached_extra_descriptor(struct libusb_device *dev,
 
 static void help(void)
 {
-	printf("Usage: dfu-util [options] ...\n"
+	printf( "Usage: dfu-util [options] ...\n"
 		"  -h --help\t\t\tPrint this help message\n"
 		"  -V --version\t\t\tPrint the version number\n"
 		"  -v --verbose\t\t\tPrint verbose debug statements\n"
-		"  -l --list\t\t\tList the currently attached DFU capable USB devices\n"
+		"  -l --list\t\t\tList the currently attached DFU capable USB devices\n");
+	printf(	"  -e --detach\t\t\tDetach the currently attached DFU capable USB devices\n"
 		"  -d --device vendor:product\tSpecify Vendor/Product ID of DFU device\n"
 		"  -p --path bus-port. ... .port\tSpecify path to DFU device\n"
 		"  -c --cfg config_nr\t\tSpecify the Configuration of DFU device\n"
 		"  -i --intf intf_nr\t\tSpecify the DFU Interface number\n"
 		"  -a --alt alt\t\t\tSpecify the Altsetting of the DFU Interface\n"
-		"\t\t\t\tby name or by number\n"
-		"  -t --transfer-size\t\tSpecify the number of bytes per USB Transfer\n"
+		"\t\t\t\tby name or by number\n");
+	printf(	"  -t --transfer-size\t\tSpecify the number of bytes per USB Transfer\n"
 		"  -U --upload file\t\tRead firmware from device into <file>\n"
 		"  -D --download file\t\tWrite firmware from <file> into device\n"
 		"  -R --reset\t\t\tIssue USB Reset signalling once we're finished\n"
@@ -554,13 +553,11 @@ static void help(void)
 
 static void print_version(void)
 {
-	printf("dfu-util %s\n\n", VERSION);
-	printf("(C) 2005-2008 by Weston Schmidt, Harald Welte and OpenMoko Inc.\n"
-	       "(C) 2010-2011 Tormod Volden (DfuSe support)\n"
-	       "This program is Free Software and has ABSOLUTELY NO WARRANTY\n\n");
-
-	printf("dfu-util does currently only support DFU version 1.0\n\n");
-
+	printf(PACKAGE_STRING "\n\n");
+	printf("Copyright 2005-2008 Weston Schmidt, Harald Welte and OpenMoko Inc.\n"
+	       "Copyright 2010-2012 Tormod Volden and Stefan Schmidt\n"
+	       "This program is Free Software and has ABSOLUTELY NO WARRANTY\n"
+	       "Please report bugs to " PACKAGE_BUGREPORT "\n\n");
 }
 
 static struct option opts[] = {
@@ -568,6 +565,7 @@ static struct option opts[] = {
 	{ "version", 0, 0, 'V' },
 	{ "verbose", 0, 0, 'v' },
 	{ "list", 0, 0, 'l' },
+	{ "detach", 0, 0, 'e' },
 	{ "device", 1, 0, 'd' },
 	{ "path", 1, 0, 'p' },
 	{ "configuration", 1, 0, 'c' },
@@ -580,15 +578,16 @@ static struct option opts[] = {
 	{ "upload", 1, 0, 'U' },
 	{ "download", 1, 0, 'D' },
 	{ "reset", 0, 0, 'R' },
-	{ "dfuse-address", 1, 0, 's' },
+	{ "dfuse-address", 1, 0, 's' }
 };
 
 enum mode {
 	MODE_NONE,
 	MODE_VERSION,
 	MODE_LIST,
+	MODE_DETACH,
 	MODE_UPLOAD,
-	MODE_DOWNLOAD,
+	MODE_DOWNLOAD
 };
 
 int main(int argc, char **argv)
@@ -597,11 +596,11 @@ int main(int argc, char **argv)
 	int num_devs;
 	int num_ifs;
 	unsigned int transfer_size = 0;
-	unsigned int host_page_size;
 	enum mode mode = MODE_NONE;
 	struct dfu_status status;
 	struct usb_dfu_func_descriptor func_dfu = {0}, func_dfu_rt = {0};
 	libusb_context *ctx;
+	struct libusb_device_descriptor desc;
 	struct dfu_file file;
 	char *alt_name = NULL; /* query alt name if non-NULL */
 	char *device_id_filter = NULL;
@@ -609,16 +608,15 @@ int main(int argc, char **argv)
 	char *end;
 	int final_reset = 0;
 	int ret;
-	int dfuse = 0;
-	unsigned int dfuse_address = 0; /* FIXME allow address to be zero? */
+	int dfuse_device = 0;
+	const char *dfuse_options = NULL;
 
-	host_page_size = getpagesize();
 	memset(dif, 0, sizeof(*dif));
 	file.name = NULL;
 
 	while (1) {
 		int c, option_index = 0;
-		c = getopt_long(argc, argv, "hVvld:p:c:i:a:t:U:D:Rs:", opts,
+		c = getopt_long(argc, argv, "hVvled:p:c:i:a:t:U:D:Rs:", opts,
 				&option_index);
 		if (c == -1)
 			break;
@@ -637,6 +635,9 @@ int main(int argc, char **argv)
 		case 'l':
 			mode = MODE_LIST;
 			break;
+		case 'e':
+			mode = MODE_DETACH;
+			break;
 		case 'd':
 			device_id_filter = optarg;
 			break;
@@ -687,15 +688,7 @@ int main(int argc, char **argv)
 			final_reset = 1;
 			break;
 		case 's':
-			dfuse = 1;
-			if (strcmp(optarg, "default")) {
-			    dfuse_address = strtoul(optarg, &end, 0);
-			    if (!dfuse_address || (*end)) {
-				fprintf(stderr, "invalid dfuse address: %s\n",
-					optarg);
-				exit(2);
-			    }
-			}
+			dfuse_options = optarg;
 			break;
 		default:
 			help();
@@ -761,10 +754,10 @@ int main(int argc, char **argv)
 
 	/* We have exactly one device. Its libusb_device is now in dif->dev */
 
-	printf("Opening DFU USB device... ");
-	libusb_open(dif->dev, &dif->dev_handle);
-	if (!dif->dev_handle) {
-		fprintf(stderr, "Cannot open device \n");
+	printf("Opening DFU capable USB device... ");
+	ret = libusb_open(dif->dev, &dif->dev_handle);
+	if (ret || !dif->dev_handle) {
+		fprintf(stderr, "Cannot open device\n");
 		exit(1);
 	}
 
@@ -776,7 +769,7 @@ int main(int argc, char **argv)
 	printf("ID %04x:%04x\n", _rt_dif.vendor, _rt_dif.product);
 
 	/* find set of quirks for this device */
-	set_quirks(_rt_dif.vendor, _rt_dif.product);
+	set_quirks(_rt_dif.vendor, _rt_dif.product, _rt_dif.bcdDevice);
 
 	/* Obtain run-time DFU functional descriptor without asking device
 	 * E.g. Freerunner does not like to be requested at this point */
@@ -813,7 +806,7 @@ int main(int argc, char **argv)
 		}
 
 		if (libusb_set_interface_alt_setting(_rt_dif.dev_handle, _rt_dif.interface, 0) < 0) {
-			fprintf(stderr, "Cannot set alt interface\n");
+			fprintf(stderr, "Cannot set alt interface zero\n");
 			exit(1);
 		}
 
@@ -825,7 +818,7 @@ int main(int argc, char **argv)
 		printf("state = %s, status = %d\n", 
 		       dfu_state_to_string(status.bState), status.bStatus);
 		if (!(quirks & QUIRK_POLLTIMEOUT))
-			usleep(status.bwPollTimeout * 1000);
+			milli_sleep(status.bwPollTimeout);
 
 		switch (status.bState) {
 		case DFU_STATE_appIDLE:
@@ -849,7 +842,7 @@ int main(int argc, char **argv)
 					fprintf(stderr, "error resetting "
 						"after detach\n");
 			}
-			sleep(2);
+			milli_sleep(2000);
 			break;
 		case DFU_STATE_dfuERROR:
 			printf("dfuERROR, clearing status\n");
@@ -870,9 +863,10 @@ int main(int argc, char **argv)
 					 _rt_dif.interface);
 		libusb_close(_rt_dif.dev_handle);
 
-		/* now we need to re-scan the bus and locate our device */
-//		if (usb_find_devices() < 2)
-//			printf("not at least 2 device changes found ?!?\n");
+		if (mode == MODE_DETACH) {
+			libusb_exit(ctx);
+			exit(0);
+		}
 
 		if (dif->flags & DFU_IFF_PATH) {
 			ret = resolve_device_path(dif);
@@ -884,7 +878,7 @@ int main(int argc, char **argv)
 			}
 			if (!ret) {
 				fprintf(stderr,
-				    "Can't resolve path after RESET?\n");
+				    "Cannot resolve path after RESET?\n");
 				exit(1);
 			}
 		}
@@ -902,9 +896,9 @@ int main(int argc, char **argv)
 		if (!get_first_dfu_device(ctx, dif))
 			exit(3);
 
-		printf("Opening USB Device...\n");
-		libusb_open(dif->dev, &dif->dev_handle);
-		if (!dif->dev_handle) {
+		printf("Opening DFU USB Device...\n");
+		ret = libusb_open(dif->dev, &dif->dev_handle);
+		if (ret || !dif->dev_handle) {
 			fprintf(stderr, "Cannot open device\n");
 			exit(1);
 		}
@@ -935,6 +929,8 @@ dfustate:
 		fprintf(stderr, "No matching DFU Interface after RESET?!?\n");
 		exit(1);
 	} else if (num_ifs > 1 ) {
+		printf("Detected interfaces after DFU transition\n");
+		list_dfu_interfaces(ctx);
 		fprintf(stderr, "We have %u DFU Interfaces/Altsettings,"
 			" you have to specify one via --intf / --alt"
 			" options\n", num_ifs);
@@ -980,7 +976,7 @@ status_again:
 	printf("state = %s, status = %d\n",
 	       dfu_state_to_string(status.bState), status.bStatus);
 	if (!(quirks & QUIRK_POLLTIMEOUT))
-		usleep(status.bwPollTimeout * 1000);
+		milli_sleep(status.bwPollTimeout);
 
 	switch (status.bState) {
 	case DFU_STATE_appIDLE:
@@ -1022,7 +1018,7 @@ status_again:
 			exit(1);
 		}
 		if (!(quirks & QUIRK_POLLTIMEOUT))
-			usleep(status.bwPollTimeout * 1000);
+			milli_sleep(status.bwPollTimeout);
 	}
 
 	/* Get the DFU mode DFU functional descriptor
@@ -1045,36 +1041,51 @@ status_again:
 		func_dfu.bcdDFUVersion = libusb_cpu_to_le16(0x0100);
 	} else if (ret < 9) {
 		printf("Error obtaining DFU functional descriptor\n");
+		printf("Please report this as a bug!\n");
 		printf("Warning: Assuming DFU version 1.0\n");
 		func_dfu.bcdDFUVersion = libusb_cpu_to_le16(0x0100);
 		printf("Warning: Transfer size can not be detected\n");
 		func_dfu.wTransferSize = 0;
 	}
+
+	if (quirks & QUIRK_FORCE_DFU11)
+		func_dfu.bcdDFUVersion = libusb_cpu_to_le16(0x0110);
+
 	printf("DFU mode device DFU version %04x\n",
 	       libusb_le16_to_cpu(func_dfu.bcdDFUVersion));
 
-	if (func_dfu.bcdDFUVersion == 0x11a)
-		dfuse = 1;
+	if (func_dfu.bcdDFUVersion == libusb_cpu_to_le16(0x11a))
+		dfuse_device = 1;
 
+	/* If not overridden by the user */
 	if (!transfer_size) {
 		transfer_size = libusb_le16_to_cpu(func_dfu.wTransferSize);
-		printf("Device returned transfer size %i\n", transfer_size);
+		if (transfer_size) {
+			printf("Device returned transfer size %i\n",
+			       transfer_size);
+		} else {
+			fprintf(stderr, "Error: Transfer size must be "
+				"specified\n");
+			exit(1);
+		}
 	}
 
-	/* if returned zero or not detected (and not user specified) */
-	if (!transfer_size) {
-		transfer_size = DEFAULT_TRANSFER_SIZE;
-		printf("Warning: Trying default transfer size %i\n",
-			transfer_size);
-	}
+#ifdef HAVE_GETPAGESIZE
+/* autotools lie when cross-compiling for Windows using mingw32/64 */
+#ifndef __MINGW32__
 	/* limitation of Linux usbdevio */
-	if (transfer_size > host_page_size) {
-		transfer_size = host_page_size;
+	if (transfer_size > getpagesize()) {
+		transfer_size = getpagesize();
 		printf("Limited transfer size to %i\n", transfer_size);
 	}
+#endif /* __MINGW32__ */
+#endif /* HAVE_GETPAGESIZE */
+
 	/* DFU specification */
-	struct libusb_device_descriptor desc;
-	libusb_get_device_descriptor(dif->dev, &desc);
+	if (libusb_get_device_descriptor(dif->dev, &desc)) {
+		fprintf(stderr, "Error: Failed to get device descriptor\n");
+		exit(1);
+	}
 	if (transfer_size < desc.bMaxPacketSize0) {
 		transfer_size = desc.bMaxPacketSize0;
 		printf("Adjusted transfer size to %i\n", transfer_size);
@@ -1082,24 +1093,30 @@ status_again:
 
 	switch (mode) {
 	case MODE_UPLOAD:
-		file.fd = open(file.name, O_WRONLY|O_CREAT|O_EXCL, 0644);
-		if (file.fd < 0) {
+		/* open for "exclusive" writing in a portable way */
+		file.filep = fopen(file.name, "ab");
+		if (file.filep == NULL) {
 			perror(file.name);
 			exit(1);
 		}
-		if (dfuse) {
+		if (ftell(file.filep)) {
+			fprintf(stderr, "%s: File exists\n", file.name);
+			fclose(file.filep);
+			exit(1);
+		}
+		if (dfuse_device || dfuse_options) {
 		    if (dfuse_do_upload(dif, transfer_size, file,
-					dfuse_address) < 0)
+					dfuse_options) < 0)
 			exit(1);
 		} else {
 		    if (dfuload_do_upload(dif, transfer_size, file) < 0)
 			exit(1);
 		}
-		close(file.fd);
+		fclose(file.filep);
 		break;
 	case MODE_DOWNLOAD:
-		file.fd = open(file.name, O_RDONLY|O_BINARY);
-		if (file.fd < 0) {
+		file.filep = fopen(file.name, "rb");
+		if (file.filep == NULL) {
 			perror(file.name);
 			exit(1);
 		}
@@ -1123,15 +1140,15 @@ status_again:
 			fprintf(stderr, "Warning: File product ID %04x does "
 				"not match device %04x\n", file.idProduct, dif->product);
 		}
-		if (dfuse || file.bcdDFU == 0x11a) {
+		if (dfuse_device || dfuse_options || file.bcdDFU == 0x11a) {
 		        if (dfuse_do_dnload(dif, transfer_size, file,
-							dfuse_address) < 0)
+							dfuse_options) < 0)
 				exit(1);
 		} else {
 			if (dfuload_do_dnload(dif, transfer_size, file) < 0)
 				exit(1);
 	 	}
-		close(file.fd);
+		fclose(file.filep);
 		break;
 	default:
 		fprintf(stderr, "Unsupported mode: %u\n", mode);
